<?xml version="1.0" encoding="ISO-8859-1" ?>
<!-- Creates XML markup from specific HTML for the Seedsavers book.

Copyright J.M. Vanel 2001 - under GNU public licence 
jmvanel@free.fr 

Worldwide Botanical Knowledge Base 
http://wwbota.free.fr/
-->

<xsl:stylesheet
 xmlns:xsl='http://www.w3.org/1999/XSL/Transform'
 xmlns:html="http://www.w3.org/1999/xhtml"
                version="1.0" >

<xsl:output indent="yes" />
<xsl:template match="text()"/>

<!-- [English name] -->
<xsl:template match="*[local-name()='p'][count(node())=1][*[local-name()='b'][not(following-sibling::*)]]">
 <name adj="english">
  <xsl:value-of select="."/>
 </name>
</xsl:template>

<!-- [Family name] -->
<xsl:template match="*[local-name()='p'][count(*)=0][not(contains(text(),' '))]"
  priority="10" >
 <family>
  <xsl:value-of select="."/>
 </family>
</xsl:template>

<!-- [Species scientific name] -->
<xsl:template match="*[local-name()='i'][not(preceding-sibling::node())]">
 <name>
  <xsl:call-template name="scientific_name" />
 </name>
</xsl:template>

<!-- Rubric title within a species
Origins, Description, Cultivation, Saving the seed, Usage, Looking around  -->
<xsl:template match="*[local-name()='b'][following-sibling::text()][not(preceding-sibling::node())]">
 <xsl:element name="{translate(normalize-space(substring-before(.,':')),' ','_')}" >
  <xsl:copy-of select="following-sibling::node()" />
  <!-- Absorb following Ordinary paragraphs: -->
  <xsl:variable name='title-id' select='generate-id()' />
  <xsl:variable name='next-medicinal'
   select='(../following-sibling::*[local-name()="p"][starts-with(.,"#")])[1]' />
 <xsl:for-each select='../following-sibling::*[local-name()="p"][child::node()[1][self::text()]][generate-id((preceding-sibling::*[local-name()="p"][*[local-name()="b"]])[position()=last()]/*[local-name()="b"]) = $title-id]
[ count($next-medicinal | following-sibling::*) = count(following-sibling::*) ]'>
   <xsl:copy-of select="." />
  </xsl:for-each>
 </xsl:element>
</xsl:template>

<!-- Ordinary paragraph, logically contained within preceeding rubric -->
<xsl:template match="*[local-name()='p'][child::node()[1][self::text()]]">
</xsl:template>

<!-- Medicinal Usage; generally follows Usage rubric -->
<xsl:template match="*[local-name()='p'][child::node()[1][self::text()][starts-with(.,'#')]]"
  priority="10" >
 <usage adj="medicinal">
  <xsl:copy-of select="node()"/>
  <!-- Absorb following Ordinary paragraphs: 
 <xsl:for-each select='following-sibling::p[child::node()[1][self::text()]][generate-id((preceding-sibling::p)[position()=last()]) = $usage-id]'>
-->
  <xsl:variable name='usage-id' select='generate-id()' />
  <xsl:variable name='next-rubric'
   select='(following-sibling::*[local-name()="p"][child::node()[1][local-name()="b"]])[1]' />

  <xsl:for-each select='following-sibling::*[local-name()="p"][child::node()[1][self::text()]][generate-id(preceding-sibling::*[local-name()="p"][starts-with(.,"#")][1]) = $usage-id]
[ count($next-rubric | following-sibling::*) = count(following-sibling::*) ]'>
    <xsl:text>
    </xsl:text>
    <xsl:copy-of select="." />
  </xsl:for-each>
 </usage>
</xsl:template>

<xsl:template name="scientific_name" >
 <xsl:variable name="s1" select="normalize-space(.)" />
 <xsl:variable name="genus" select="substring-before($s1,' ')" />
 <xsl:variable name="sa" select="substring-after ($s1,' ')" />
 <xsl:variable name="hepithet" >
  <xsl:choose>
   <xsl:when test= "substring-before($sa,' ')" >
    <xsl:value-of select="substring-before($sa,' ')" />
   </xsl:when>
   <xsl:otherwise>
    <xsl:value-of select="$sa" />
   </xsl:otherwise>
  </xsl:choose>
 </xsl:variable>
 <genus>
    <xsl:value-of select="$genus" />
 </genus>
 <xsl:text>
 </xsl:text>
 <hepithet>
    <xsl:value-of select="$hepithet" />
 </hepithet>
 <xsl:value-of select="substring-after ($s1, $hepithet)" />
</xsl:template>

<xsl:template match="/">
 <book>
  <xsl:apply-templates />
 </book>
</xsl:template>

</xsl:stylesheet>
