<?xml version="1.0" encoding="ISO-8859-1" ?>
<!-- 

Copyright J.M. Vanel 2001 - under GNU public licence 
jmvanel@free.fr 

Worldwide Botanical Knowledge Base 
http://wwbota.free.fr/

$Header: /usr/local/cvsroot/XMLPublication/kernel/merge-items.xslt,v 1.1 2002/02/01 14:20:34 jmv Exp $ -->

<xsl:stylesheet xmlns:xsl='http://www.w3.org/1999/XSL/Transform'
                xmlns:saxon='http://icl.com/saxon'
                exclude-result-prefixes="saxon"
xmlns:_xmlpub="http://xmlpublication.org/"
                version="1.0" >

  <xsl:import href='make-hyperlinked-index-merge.xslt'/>
  <xsl:import href='make-index-by-rubrics-impl.xslt'/>
  <xsl:import href='make-words-list.xslt'/>
  <xsl:import href="filter-words-list.xslt" />
  <xsl:output indent='yes' />

<xsl:param name="thesaurus"  select="'USER_SETTINGS/thesaurus.xml'"/>
<xsl:variable name='thesaurus_doc'  select="document($thesaurus)" />
<xsl:variable name='merge-items' select="$thesaurus_doc/*/merge-items" />
<xsl:variable name='according-to-label' select="$merge-items/@according-to-label" />
<xsl:variable name='master' select="/" />
<xsl:variable name='xpath' >
   <xsl:value-of select="$thesaurus_doc/*/rubric [@label = $according-to-label] / @xpath" />
</xsl:variable>

<xsl:template match="/" >

  <xsl:choose>
  <xsl:when test="$merge-items" >
     <xsl:message> Searching words for key rubric <xsl:value-of select="$according-to-label" /> ... </xsl:message>
     <xsl:message> DEBUG : Searching words for key rubric : $xpath= <xsl:value-of select="$xpath" /> ... </xsl:message>

     <xsl:variable name='rubric-content'  >
       <xsl:apply-templates select="saxon:evaluate(concat('$master//', $xpath ) )" mode='add-spaces' />
     </xsl:variable>

     <xsl:variable name='rubric-values'  >
       <xsl:call-template name="filter-words-list">
        <xsl:with-param name="nodes" >
         <xsl:call-template name="make-words-list">
          <xsl:with-param name="string" >
           <xsl:apply-templates select="$rubric-content" mode= 'suppress-unwanted-chars' />
          </xsl:with-param>
         </xsl:call-template>
        </xsl:with-param>
       </xsl:call-template>
     </xsl:variable>
     <xsl:message> Words for rubric <xsl:value-of select="$according-to-label" /> :
       <xsl:copy-of select="$rubric-values" />
     </xsl:message>
    <!-- <xsl:apply-templates select='/' mode='make-index-by-rubrics' /> -->
    <xsl:apply-templates select='/' mode='merge-items' >
      <xsl:with-param name="rubric-values" select="$rubric-values" />
    </xsl:apply-templates>
  </xsl:when>
  <xsl:otherwise>
     <xsl:message> !!! No merge-items element in thesaurus.xml; see documentation ... </xsl:message>
  </xsl:otherwise>
  </xsl:choose>
</xsl:template>

<xsl:template match='/' mode='merge-items'>
  <xsl:param name="rubric-values" />
  <xsl:message> DEBUG :  $rubric-values= <xsl:copy-of select="$rubric-values" /> </xsl:message>
  <xsl:variable name="apos" >"</xsl:variable>

  <xsl:variable name='rubrics' >
    <xsl:value-of select="$thesaurus_doc/*/*[name()='rubric']" />
  </xsl:variable>
 <items>

  <xsl:for-each select="$rubric-values/*" >
     <xsl:message> DEBUG :  $rubric-values/*= <xsl:copy-of select="." /> </xsl:message>
     <!-- DEBUG :  $rubric-values/*= <xsl:copy-of select="." /> -->
     <xsl:variable name='rubric-value' select="." />

     <!-- query all items whose rubric $xpath contains $rubric-value -->
     <xsl:variable name="query" select="concat('$master/*/*',
                                       '[contains(', $xpath, ',', $apos,$rubric-value,$apos, ')]' )" />
     <xsl:message> DEBUG: merge-items: $query= <xsl:copy-of select="$query" /> </xsl:message>
     <xsl:variable name="raw-items" select="saxon:evaluate($query)" />
     <xsl:message> DEBUG count($raw-items)=<xsl:copy-of select="count($raw-items)" /> </xsl:message>

     <xsl:variable name='items' >
     <p class='h1'>
      <h1><xsl:value-of select="$rubric-value" /></h1>

      <xsl:for-each select="$thesaurus_doc/*/rubric" >
       <xsl:variable name='rubric' select="." />

       <xsl:variable name='merged-rubric'>
        <xsl:for-each select="$raw-items" >
         <xsl:variable name='item' select="." />
         <!--
          <xsl:variable name="query2" select="concat('$item/* [local-name()=', $apos, $rubric/@name, $apos, ']' )" />
         -->
          <xsl:variable name="query2" select="concat( '$item/', $rubric/@xpath )" />
          <xsl:message> DEBUG : $query2= <xsl:value-of select="$query2" /> </xsl:message>
          <xsl:variable name="raw-rubrics" select="saxon:evaluate($query2)" />
          <xsl:message> DEBUG merge-items: $item/*[1] = <xsl:copy-of select="./*[1]" /> </xsl:message>
          <xsl:message> DEBUG : $raw-rubrics= <xsl:copy-of select="$raw-rubrics" /> </xsl:message>
            <xsl:if test="normalize-space($raw-rubrics/node())" >
              <xsl:copy-of select="$raw-rubrics/node()[not(local-name()='h2')]" />
              <_xmlpub:source><xsl:value-of select="@_xmlpub:source-URI" /></_xmlpub:source>
            </xsl:if>
        </xsl:for-each>
       </xsl:variable>

       <p class='h2'>
         <h2><xsl:value-of select="$rubric/@label" /></h2>
         <xsl:copy-of select='$merged-rubric' />
       </p>
      </xsl:for-each>
     </p>
     </xsl:variable>
     <xsl:copy-of select="$items" />
  </xsl:for-each>
 </items>
</xsl:template>

<!-- suppress unwanted characters; this is applied before making index words list for each rubric. -->
<xsl:template match='text()' mode='suppress-unwanted-chars'>
 <xsl:variable name='search'  >.*"[]:,;?()&amp;&#160;</xsl:variable>
 <xsl:variable name='replace' >             </xsl:variable>
 <xsl:value-of select="translate(., $search, $replace )" />
</xsl:template>

<xsl:template match='*' mode='add-spaces'>
   <xsl:copy-of select="." />
   <xsl:text>
   </xsl:text>
</xsl:template>

</xsl:stylesheet>
