<?xml version="1.0" encoding="UTF-8"?>
<!-- edited with XML Spy v3.0.7 NT (http://www.xmlspy.com) by Arnold deVos (Langdale Consultants) -->
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:rdfs="http://www.w3.org/2000/01/rdf-schema#" xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#" xmlns:cims="http://iec.ch/TC57/1999/rdf-schema-extensions-19990926#" xmlns:xp="http://au.com.Langdale/2000/Xpetal">
	<xsl:output method="xml" version="1.0" encoding="UTF-8" indent="yes"/>
	<xsl:param name="pname">Nerc</xsl:param>
	<xsl:param name="cimns"/>
	<xsl:variable name="preref">
		<xsl:value-of select="$cimns"/>
		<xsl:if test="$cimns = ''">#</xsl:if>
	</xsl:variable>
	<xsl:variable name="idattr">
		<xsl:choose>
			<xsl:when test="$cimns">about</xsl:when>
			<xsl:otherwise>ID</xsl:otherwise>
		</xsl:choose>
	</xsl:variable>
	<xsl:template match="xp:Xpetal">
		<rdf:RDF>
			<xsl:apply-templates/>
		</rdf:RDF>
	</xsl:template>
	<xsl:template match="text()"/>
	<xsl:template match="xp:Class_Category">
		<xsl:variable name="category" select="translate(normalize-space(xp:arg1),' ','_')"/>
		<cims:ClassCategory>
			<xsl:attribute name="rdf:{$idattr}"><xsl:value-of select="concat($cimns,$category)"/></xsl:attribute>
			<xsl:call-template name="description"/>
		</cims:ClassCategory>
		<xsl:apply-templates>
			<xsl:with-param name="category" select="$category"/>
		</xsl:apply-templates>
	</xsl:template>
	<xsl:template name="class">
		<xsl:param name="category"/>
		<rdfs:Class>
			<xsl:call-template name="ident"/>
			<xsl:if test="$category">
				<cims:belongsToCategory>
					<xsl:attribute name="rdf:resource"><xsl:value-of select="$preref"/><xsl:value-of select="$category"/></xsl:attribute>
				</cims:belongsToCategory>
			</xsl:if>
			<xsl:for-each select="xp:Inheritance_Relationship">
				<rdfs:subClassOf rdf:resource="{$preref}{xp:referent}"/>
			</xsl:for-each>
		</rdfs:Class>
	</xsl:template>
	<xsl:template match="xp:Class">
		<xsl:param name="category"/>
		<xsl:call-template name="class">
			<xsl:with-param name="category" select="$category"/>
		</xsl:call-template>
		<xsl:variable name="class" select="xp:arg1"/>
		<xsl:for-each select="xp:ClassAttribute">
			<rdf:Property>
				<xsl:call-template name="ident">
					<xsl:with-param name="prefix" select="$class"/>
				</xsl:call-template>
				<rdfs:domain rdf:resource="{$preref}{$class}"/>
				<xsl:choose>
					<xsl:when test="xp:stereotype='Enumeration'">
						<rdfs:range rdf:resource="{$preref}{xp:type}"/>
					</xsl:when>
					<xsl:otherwise>
						<cims:dataType rdf:resource="{$preref}{xp:type}"/>
					</xsl:otherwise>
				</xsl:choose>
			</rdf:Property>
		</xsl:for-each>
	</xsl:template>
	<xsl:template match="xp:Class[xp:stereotype='Enumeration']">
		<xsl:param name="category"/>
		<xsl:call-template name="class">
			<xsl:with-param name="category" select="$category"/>
		</xsl:call-template>
		<xsl:variable name="class" select="xp:arg1"/>
		<xsl:for-each select="xp:ClassAttribute">
			<rdf:Description>
				<xsl:call-template name="ident">
					<xsl:with-param name="prefix" select="$class"/>
				</xsl:call-template>
				<rdf:type rdf:resource="{$preref}{$class}"/>
			</rdf:Description>
		</xsl:for-each>
	</xsl:template>
	<xsl:template match="xp:Class[xp:stereotype='Primitive']">
		<xsl:param name="category"/>
		<xsl:call-template name="class">
			<xsl:with-param name="category" select="$category"/>
		</xsl:call-template>
	</xsl:template>
	<xsl:template match="xp:Association">
		<xsl:apply-templates select="xp:Role[1]">
			<xsl:with-param name="class" select="xp:Role[2]/xp:referent"/>
			<xsl:with-param name="inverse" select="xp:Role[2]/xp:arg1"/>
		</xsl:apply-templates>
		<xsl:apply-templates select="xp:Role[2]">
			<xsl:with-param name="class" select="xp:Role[1]/xp:referent"/>
			<xsl:with-param name="inverse" select="xp:Role[1]/xp:arg1"/>
		</xsl:apply-templates>
	</xsl:template>
	<xsl:template name="ident">
		<xsl:param name="prefix"/>
		<xsl:variable name="local">
			<xsl:value-of select="$prefix"/>
			<xsl:if test="$prefix">
				<xsl:text>.</xsl:text>
			</xsl:if>
			<xsl:value-of select="xp:arg1"/>
		</xsl:variable>
		<xsl:attribute name="rdf:{$idattr}"><xsl:value-of select="concat($cimns,$local)"/></xsl:attribute>
		<xsl:call-template name="description"/>
	</xsl:template>
	<xsl:template name="description">
		<rdfs:label xml:lang="en">
			<xsl:value-of select="xp:arg1"/>
		</rdfs:label>
		<rdfs:comment>
			<xsl:value-of select="xp:documentation"/>
		</rdfs:comment>
		<xsl:apply-templates select="xp:Attribute"/>
	</xsl:template>
	<xsl:template match="xp:Attribute[xp:name=concat($pname,'Profile')]">
		<cims:profile>
			<xsl:value-of select="$pname"/>
		</cims:profile>
	</xsl:template>
	<xsl:template match="xp:Role">
		<xsl:param name="class"/>
		<xsl:param name="inverse"/>
		<rdf:Property>
			<xsl:call-template name="ident">
				<xsl:with-param name="prefix" select="$class"/>
			</xsl:call-template>
			<xsl:apply-templates select="../xp:Attribute"/>
			<rdfs:domain rdf:resource="{$preref}{$class}"/>
			<rdfs:range rdf:resource="{$preref}{xp:referent}"/>
			<xsl:if test="xp:client_cardinality">
				<cims:multiplicity>
					<xsl:attribute name="rdf:resource"><xsl:text>http://iec.ch/TC57/1999/rdf-schema-extensions-19990926#M:</xsl:text><xsl:choose><xsl:when test="xp:client_cardinality/xp:cardinality='0..*'"><xsl:text>0..n</xsl:text></xsl:when><xsl:when test="xp:client_cardinality/xp:cardinality='1..*'"><xsl:text>1..n</xsl:text></xsl:when><xsl:otherwise><xsl:value-of select="xp:client_cardinality/xp:cardinality"/></xsl:otherwise></xsl:choose></xsl:attribute>
				</cims:multiplicity>
			</xsl:if>
			<xsl:if test="xp:is_aggregate='TRUE'">
				<cims:isAggregate>true</cims:isAggregate>
			</xsl:if>
			<cims:inverseRoleName>
				<xsl:attribute name="rdf:resource"><xsl:value-of select="$preref"/><xsl:value-of select="xp:referent"/><xsl:text>.</xsl:text><xsl:value-of select="$inverse"/></xsl:attribute>
			</cims:inverseRoleName>
		</rdf:Property>
	</xsl:template>
</xsl:stylesheet>
