<?xml version="1.0" encoding="UTF-8"?>
<!-- edited with XML Spy v3.0.7 NT (http://www.xmlspy.com) by Arnold deVos (Langdale Consultants) -->
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:rdfs="http://www.w3.org/2000/01/rdf-schema#" xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#" xmlns:cims="http://iec.ch/TC57/1999/rdf-schema-extensions-19990926#" xmlns:xsd="http://www.w3.org/2000/10/XMLSchema" >
	<xsl:output method="xml" version="1.0" encoding="UTF-8" indent="yes"/>
	
	<!-- the namespace for the elements and types defined in the generated schema. 
		unfortunately, this must line up with the statically declared namespace prefix (XSLT limitation!)-->
	<xsl:param name="schemans">http://iec.ch/TC57/2001/CIM-schema-cim10</xsl:param>
	<xsl:param name="cimprefix">cim:</xsl:param>
	
	<!-- ignore all text nodes -->
	<xsl:template match="text()"/>

	<!-- process enumerated values -->
	<xsl:template match="rdf:Description">
		<xsl:variable name="id">
			<xsl:value-of select="@rdf:ID"/>
			<xsl:value-of select="substring-after(@rdf:about,'#')"/>
		</xsl:variable>
		<xsd:enumeration value="{$id}">
		</xsd:enumeration>
	</xsl:template>
	
	<!-- all properties become global elements -->
	<xsl:template match="rdf:Property">
		<xsl:variable name="id">
			<xsl:value-of select="@rdf:ID"/>
			<xsl:value-of select="substring-after(@rdf:about,'#')"/>
		</xsl:variable>
		<xsl:variable name="type">
			<xsl:value-of select="$cimprefix"/>
			<xsl:value-of select="substring-after(cims:dataType/@rdf:resource, '#')"/>
			<xsl:value-of select="substring-after(rdfs:range/@rdf:resource, '#')"/>
		</xsl:variable>
		<xsd:element name="{$id}" type="{$type}">
			<xsd:annotation><xsd:documentation><xsl:value-of select="rdfs:comment"/></xsd:documentation></xsd:annotation>
		</xsd:element>
	</xsl:template>


	<!-- ignore these properties which are in error in the schema -->	
	<xsl:template match="rdf:Property[@rdf:ID='ConductorType.sheathResistance']"/>
	<xsl:template match="rdf:Property[@rdf:ID='ConductorType.sheathReactance']"/>
	<xsl:template match="rdf:Property[@rdf:ID='InspectionRoutine.Assets']"/>
	<xsl:template match="rdf:Property[@rdf:ID='AssetPropertySet.propertyID']"/>
	<xsl:template match="rdf:Property[@rdf:ID='AssetRatingSet.ratingID']"/>
	<xsl:template match="rdf:Property[@rdf:ID='Asset.InspectionRoutines']"/>
	<xsl:template match="rdf:Property[@rdf:ID='InspectionRoutine.id']"/>
	<xsl:template match="rdf:Property[@rdf:ID='MaintenanceProcedure.id']"/>
	<xsl:template match="rdf:Property[@rdf:ID='Schedule.dueDate']"/>
	<xsl:template match="rdf:Property[@rdf:ID='Schedule.scheduledStartDate']"/>
	<xsl:template match="rdf:Property[@rdf:ID='Schedule.plannedStartDate']"/>
	<xsl:template match="rdf:Property[@rdf:ID='Schedule.plannedFinishDate']"/>
	<xsl:template match="rdf:Property[@rdf:ID='Schedule.earliestStartDate']"/>
	<xsl:template match="rdf:Property[@rdf:ID='Schedule.latestFinishtDate']"/>
	<xsl:template match="rdf:Property[@rdf:ID='Schedule.actualStartDate']"/>
	<xsl:template match="rdf:Property[@rdf:ID='Schedule.actualFinishtDate']"/>
	<xsl:template match="rdf:Property[@rdf:ID='SwitchAsset.gang']"/>
	<xsl:template match="rdf:Property[@rdf:ID='SwitchAsset.remote']"/>
	<xsl:template match="rdf:Property[starts-with(@rdf:ID,'Volume.')]"/>
	<xsl:template match="rdf:Property[@rdf:ID='Streetlight.rating']"/>
	<xsl:template match="rdf:Property[@rdf:ID='Permit.date']"/>
	<xsl:template match="rdf:Property[@rdf:ID='LocationGrant.date']"/>
	<xsl:template match="rdf:Property[contains(@rdf:ID,'$')]"/>

	<!-- an rdf class could be a datatype or a set of resources (in this schema anyway) -->
	<xsl:template match="rdfs:Class">
		<xsl:choose>
			<xsl:when test="substring-after(cims:belongsToCategory/@rdf:resource,'#')='Domain'">
				<xsl:call-template name="datatype"/>
			</xsl:when>
			<xsl:otherwise>
				<xsl:call-template name="class"/>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	
	<!-- most classes become URI references -->
	<xsl:template name="class">
		<xsl:variable name="id">
			<xsl:value-of select="@rdf:ID"/>
			<xsl:value-of select="substring-after(@rdf:about,'#')"/>
		</xsl:variable>
		<xsd:simpleType name="{$id}">
			<xsd:annotation><xsd:documentation><xsl:value-of select="rdfs:comment"/></xsd:documentation></xsd:annotation>
			<xsd:restriction base="xsd:QName">
			</xsd:restriction>
		</xsd:simpleType>
	</xsl:template>
	
	<!-- domain classes become simpleTypes -->
	<xsl:template name="datatype">
		<xsl:variable name="ref">
			<xsl:choose>
				<xsl:when test="@rdf:ID"><xsl:value-of select="concat('#',@rdf:ID)"/></xsl:when>
				<xsl:otherwise><xsl:value-of select="@rdf:about"/></xsl:otherwise>
			</xsl:choose>
		</xsl:variable>
		<xsl:variable name="id" select="substring-after($ref,'#')"/>
		<xsd:simpleType name="{$id}">
			<xsd:annotation><xsd:documentation><xsl:value-of select="rdfs:comment"/></xsd:documentation></xsd:annotation>
			<xsd:restriction base="xsd:string">
				<xsl:apply-templates select="/rdf:RDF/rdf:Description[rdf:type/@rdf:resource=$ref]"/>
			</xsd:restriction>
		</xsd:simpleType>
	</xsl:template>
	
	<xsl:template name="hardwired-types">
		<xsd:simpleType name="IntervalTime"><xsd:restriction base="xsd:date"/></xsd:simpleType> 
		<xsd:simpleType name="AbsoluteDate"><xsd:restriction base="xsd:date"/></xsd:simpleType> 
		<xsd:simpleType name="SerialNumber"><xsd:restriction base="xsd:string"/></xsd:simpleType> 
		<xsd:simpleType name="String"><xsd:restriction base="xsd:string"/></xsd:simpleType> 
		<xsd:simpleType name="string"><xsd:restriction base="xsd:string"/></xsd:simpleType> 
		<xsd:simpleType name="Float"><xsd:restriction base="xsd:float"/></xsd:simpleType> 
		<xsd:simpleType name="Integer"><xsd:restriction base="xsd:integer"/></xsd:simpleType> 
	</xsl:template>

	<!-- the document element -->
	<xsl:template match="rdf:RDF">
		<xsd:schema xmlns:cim="http://iec.ch/TC57/2001/CIM-schema-cim10">
				<xsl:if test="$schemans">
					<xsl:attribute name="targetNamespace"><xsl:value-of select="$schemans"/></xsl:attribute>
				</xsl:if>
				<xsl:call-template name="hardwired-types"/>
				<xsl:apply-templates select="rdfs:Class|rdf:Property"/>
		</xsd:schema>
	</xsl:template>
</xsl:stylesheet>
