/* Copyright (c) 2002. Jean-Marc Vanel. All rights reserved.
 * This code is provided AS-IS, WITHOUT ANY WARRANTY either expressed or implied.
 $Id: HTMLHeaderWrapperXMLFilter.java,v 1.1.1.1 2002/05/08 07:56:07 jmv Exp $
*/

package org.xmlpublication;

import java.io.IOException;

import org.xml.sax.XMLReader;
import org.xml.sax.XMLFilter;
import org.xml.sax.InputSource;
import org.xml.sax.Locator;
import org.xml.sax.Attributes;
import org.xml.sax.EntityResolver;
import org.xml.sax.DTDHandler;
import org.xml.sax.ContentHandler;
import org.xml.sax.ErrorHandler;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import org.xml.sax.SAXNotSupportedException;
import org.xml.sax.SAXNotRecognizedException;
import org.xml.sax.helpers.*;
import javax.xml.parsers.*;

/** HTML Header Wrapper XMLFilter
 * 
<p>This class is designed to sit between an {@link org.xml.sax.XMLReader
 * XMLReader} and the client application's event handlers. It passes requests 
 * up to the reader and most events
 * on to the handlers unmodified, except that:
 * <hn>...</hn> lots of sibling tags
 * gets transformed into:
 * <div class="hn"><hn>...</hn>lots of sibling tags</div>
 
 * Subclasses can override
 * specific methods to modify the event stream or the configuration
 * requests as they pass through.</p>
 *
 * TODO : operation for this feature should be idempotent (an algebraic operation x is said idempotent if x is such that xx=x).
 * @author Jean-Marc Vanel
 */
public class HTMLHeaderWrapperXMLFilter extends XMLFilterImpl
// public class HTMLHeaderWrapperXMLFilter extends org.xml.sax.helpers.DefaultHandler
    implements XMLFilter, ContentHandler
{
  static final String wrapperHTMLElement = "div"; // WRAPPERTAG in XSLT transforms
  String HTMLuri="";
  short lastOpenedHeaderLevel=0;


    ////////////////////////////////////////////////////////////////////
    // Constructors.
    ////////////////////////////////////////////////////////////////////

public HTMLHeaderWrapperXMLFilter ()
    {
	super();
    }


    ////////////////////////////////////////////////////////////////////
    // Other functions.
    ////////////////////////////////////////////////////////////////////

    public short HTMLHeaderLevel(String s) {
      short level=0;
      if ( ( s.startsWith("h") ||
             s.startsWith("H")    ) &&
             s.length() == 2 ) {
        level = (short)Integer.parseInt(s.substring(1));
      }      
      return level;
    }

    void closeLevelIfNecessary(short level)
	throws SAXException
    {
	  if ( level > 0 ) {
            short levelsToclose = (short)(lastOpenedHeaderLevel - level + 1);
            for ( short i=0; i<levelsToclose; i++ ) {
	      getContentHandler().endElement(HTMLuri, wrapperHTMLElement, wrapperHTMLElement );
	    }
	  }
    }


    ////////////////////////////////////////////////////////////////////
    // Implementation of org.xml.sax.ContentHandler.
    ////////////////////////////////////////////////////////////////////

    /** Filter a start element event.
     *
     * @param uri The element's Namespace URI, or the empty string.
     * @param localName The element's local name, or the empty string.
     * @param qName The element's qualified (prefixed) name, or the empty
     *        string.
     * @param atts The element's attributes.
     * @exception org.xml.sax.SAXException The client may throw
     *            an exception during processing.
     * @see org.xml.sax.ContentHandler#startElement
     */
    public void startElement (String uri, String localName, String qName,
			      Attributes atts)
	throws SAXException
    {
	if (getContentHandler() != null) {
          short level= HTMLHeaderLevel(localName);
          // System.out.print ("level lastOpenedHeaderLevel: "); System.out.print( level ); System.out.print (", "); System.out.println( lastOpenedHeaderLevel);
          closeLevelIfNecessary(level);
	  if ( level > 0 ) {
              AttributesImpl classAttrib = new AttributesImpl();
              classAttrib.addAttribute( "", "class", "class", ""/*type*/, "h" + (new Integer(level)).toString() );
	      getContentHandler().startElement(HTMLuri, wrapperHTMLElement, wrapperHTMLElement, classAttrib );
	      lastOpenedHeaderLevel = level;
	  }
	  getContentHandler().startElement(uri, localName, qName, atts);
	}
    }

    /** Filter an end element event. */
    public void endElement(String namespaceURI,
                       String localName,
                       String qName)
                throws SAXException {

	if (getContentHandler() != null) {
	 if ( localName.equals("body") ||
              localName.equals("BODY")    ) {
          closeLevelIfNecessary((short)1);
         }
         getContentHandler().endElement( namespaceURI, localName, qName);
        }
    }

    /**
     * Filter an end document event.
     *
     * @exception org.xml.sax.SAXException The client may throw
     *            an exception during processing.
     * @see org.xml.sax.ContentHandler#endDocument
     */
    public void endDocument ()
	throws SAXException
    {
	if (getContentHandler() != null) {
          // closeLevelIfNecessary((short)1);
          getContentHandler().endDocument();
	}
        System.out.println( "HTMLHeaderWrapperXMLFilter: endDocument()" );
    }


  public static void main (String[] args) throws Exception {
    System.out.println( args[0]);
    SAXParserFactory factory = SAXParserFactory.newInstance();
    // factory.setValidating(true);
    SAXParser parser = factory.newSAXParser();

    XMLFilterImpl filter = new HTMLHeaderWrapperXMLFilter(); 
    filter.setParent(parser.getXMLReader());
    filter.setContentHandler(new Writer());
    filter.parse(args[0]);

    // OK parser.parse(args[0], new Writer());
  }
}
